from openbci import cyton as board
import time
import random
from pydub import AudioSegment
from pydub.playback import play
from sys import exit as exit
import platform

from wheelchair import save_file as filename
from wheelchair import samples_per_trial
from wheelchair import lab

##############################################################################
##																			##
##                            GLOBAL VARIABLES                              ##
## 																			##
##############################################################################

# Create labels list, random shuffle, insert 5 beeps at the beginning & append 'EXIT' at the end
samples_per_action = 15
labels = lab * samples_per_action
random.shuffle(labels)

for i in range(0,5):
	labels.insert(0,"beep")
labels.append('EXIT')

# Create an iterator over label array
iter = 0
# Create a counter for the number of datapoints
cont = 0

def connect():
	"""
FUNCTION:		connect()
INPUT:			None
OUTPUT:			Call to acquire_save(connect)
DESCRIPTION:	Creates the connection with OpenBCI Cyton Board
				and sets the configuration
	"""

# CONNECTION PARAMETERS
	# Check for OS system
	if platform.system() == 'Darwin':	# Mac OSX
		port = '/dev/tty.usbserial-DM01HL5T'
	else:								# Raspberry Pi
		port = 'dev/ttyUSB0'
	
	baud = 115200
	filter_data = False
	scaled_output = True
	daisy = False
	aux = False
	impedance = False
	log = False
	timeout = None

# CONNECTION ESTABLISHMENT
	connection = board.OpenBCICyton(port=port,
									baud=baud,
									filter_data=filter_data,
									scaled_output=scaled_output,
									daisy=daisy,
									aux=aux,
									impedance=impedance,
									log=log,
									timeout=timeout)

# CONFIGURATION
	time.sleep(1)
	# Turn Off channels [1, 3, 4, 5, 6, 8]
	connection.ser.write(b'1')
	connection.ser.write(b'3')
	connection.ser.write(b'4')
	connection.ser.write(b'5')
	connection.ser.write(b'6')
	connection.ser.write(b'8')
	# For channels 2 & 7: 
	# Power 'On'
	# Gain set to 'x8'
	# Input type to 'normal'
	# 'Remove' channel from bias
	# 'disconnect' channel from SRB2
	# 'disconnect' all channels from SRB1
	time.sleep(1)
	connection.ser.write(b'x2040000X')
	time.sleep(1)
	connection.ser.write(b'x7040000X')
	time.sleep(1)

	acquire_save(connection)


def acquire_save(connection):
	"""
# FUNCTION:	 	acquire_save(connection)
# INPUT: 		connection object
# OUTPUT: 		Loop over handle_streamed_data
# DESCRIPTION:	Streams the data from Cyton Board
	"""

	print("Dongle Connected with board")   
	time.sleep(1)
	print("Dongle streaming started...")

	connection.print_register_settings()  
	connection.start_streaming(handle_streamed_data)
	
	connection.loop()

def handle_streamed_data(sample):
	"""
# FUNCTION:	 	handle_streamed_data(sample)
# INPUT: 		Sample object
# OUTPUT: 		None
# DESCRIPTION:	Reads the data and saves it on a .txt file,
#				also plays audio files with actions when its necessary
	"""

	global labels, iter, cont, file, samples_per_trial

	tmp_lab = labels[iter]

	# Open file were we are going to save the data
	file = open(filename, 'a')

	if cont == 0:

		if tmp_lab == 'EXIT':
			file.close()
			print("**********************************")
			print("**********************************")
			print("**   TRAINNING BLOCK FINISHED   **")
			print("**********************************")
			print("**********************************")
			time.sleep(1)
			exit()
		else:
			song_name = "./audio/"+tmp_lab+".mp3"
			song = AudioSegment.from_mp3(song_name)
			play(song)
			cont += 1
			sample.channel_data.insert(0,tmp_lab)
			str_chn_dta =  ' '.join([str(sample.channel_data)])
			file.write(str_chn_dta)
			file.write('\n')
			

	elif cont < samples_per_trial-1:
		cont += 1
		sample.channel_data.insert(0,tmp_lab)
		str_chn_dta =  ' '.join([str(sample.channel_data)])
		file.write(str_chn_dta)
		file.write('\n')
	else:
		cont = 0
		iter +=1
		sample.channel_data.insert(0,tmp_lab)
		str_chn_dta =  ' '.join([str(sample.channel_data)])
		file.write(str_chn_dta)
		file.write('\n')


