import numpy as np
from scipy import stats

def features(trial_hor, trial_ver):
	"""
# FUNCTION:	 	features(trial_hor, trial_ver)
# INPUT: 		Horizontal & Vertical components of each trial
# OUTPUT: 		Matrix with feature values of horizontal & vertical
#				components of each trial
# DESCRIPTION:	Extracts the features for each trial
	"""

	num_feat = 24 # Number of Features (12 vertical and 12 horizontal)
	features = np.zeros((len(trial_hor), num_feat))

	
	for iter in range(0, len(trial_hor)):
		
		features[iter][0] = np.min(trial_hor[iter]) # Horizontal Min 
		features[iter][1] = np.min(trial_ver[iter]) # Vertical Min
		
		features[iter][2] = np.max(trial_hor[iter]) # Horizontal Max
		features[iter][3] = np.max(trial_ver[iter]) # Vertical Max
		
		features[iter][4] = np.median(trial_hor[iter]) # Horizontal Median
		features[iter][5] = np.median(trial_ver[iter]) # Vertical Median

		features[iter][6] = float(stats.mode(trial_hor[iter])[0]) # Horizontal Mode
		features[iter][7] = float(stats.mode(trial_ver[iter])[0]) # Vertical Mode

		features[iter][8] = stats.kurtosis(trial_hor[iter], nan_policy='omit') # Horizontal Kurtosis
		features[iter][9] = stats.kurtosis(trial_ver[iter], nan_policy='omit') # Vertical Kurtosis

		features[iter][10] = int(np.where(trial_hor[iter] == np.max(trial_hor[iter]))[0][0]) # Horizontal Max position
		features[iter][11] = int(np.where(trial_ver[iter] == np.max(trial_ver[iter]))[0][0]) # Vertical Max position

		features[iter][12] = int(np.where(trial_hor[iter] == np.min(trial_hor[iter]))[0][0]) # Horizontal Min position
		features[iter][13] = int(np.where(trial_ver[iter] == np.min(trial_ver[iter]))[0][0]) # Vertical Min position
		
		features[iter][14] = sum(abs(trial_hor[iter])) # Horizontal Area under the curve
		features[iter][15] = sum(abs(trial_ver[iter])) # Vertical Area under the curve

		features[iter][16] = np.percentile(trial_hor[iter], 25) # Horizontal First quartile
		features[iter][17] = np.percentile(trial_ver[iter], 25) # Vertical First quartile

		features[iter][18] = np.percentile(trial_hor[iter], 75) # Horizontal Third quartile
		features[iter][19] = np.percentile(trial_ver[iter], 75) # Vertical Third quartile

		features[iter][20] = np.percentile(trial_hor[iter], 75) - np.percentile(trial_hor[iter], 25) # Horizontal Interquartile range
		features[iter][21] = np.percentile(trial_ver[iter], 75) - np.percentile(trial_ver[iter], 25) # Vertical Interquartile range
		

# Calculate derivative & derivative slope
		der_hor = np.zeros(len(trial_hor[iter]))
		der_ver = np.zeros(len(trial_ver[iter]))

		pos_min_der_hor = []
		pos_max_der_hor = []
		pos_min_der_ver = []
		pos_max_der_ver = []

		min_der_hor = 0
		max_der_hor = 0
		min_der_ver = 0
		max_der_ver = 0

		for j in range(5, len(trial_hor[iter])):
			der_hor[j] = (trial_hor[iter][j] - trial_hor[iter][j-5]) / 5
			der_ver[j] = (trial_ver[iter][j] - trial_ver[iter][j-5]) / 5
		
		min_der_hor = np.min(der_hor)
		max_der_hor = np.max(der_hor)
		min_der_ver = np.min(der_ver)
		max_der_ver = np.max(der_ver)

		pos_min_der_hor = int(np.where(der_hor == min_der_hor)[0][0])
		pos_max_der_hor = int(np.where(der_hor == max_der_hor)[0][0])
		pos_min_der_ver = int(np.where(der_ver == min_der_ver)[0][0])
		pos_max_der_ver = int(np.where(der_ver == max_der_ver)[0][0])

		features[iter][22] = abs(max_der_hor - min_der_hor) / (pos_max_der_hor - pos_min_der_hor)# Horizontal Derivative Slope
		features[iter][23] = abs(max_der_ver - min_der_ver) / (pos_max_der_ver - pos_min_der_ver)# Vertical Derivative Slope
		

	return features