import numpy as np
from scipy import signal
from scipy.signal import butter, lfilter

import read_function as read
import split_into_trials as tri


def filter(files, samples_per_trial):
	"""
# FUNCTION:	 	filter(files)
# INPUT: 		List of files to filter and number of samples per trial
# OUTPUT: 		Vertical & horizontal components of the data divided by trials
#					and a list containing the label of each trial
# DESCRIPTION:	Applies a Butterworth filter and a smoothing filter over the data
#					in each file. Then calls to_trials() to normalize and split into trials
	"""
	lowcut = 0.03
	highcut = 12
	fs = 250
	order = 2

	hor_tmp = []
	ver_tmp = []
	labels_tmp = []
	hor_bnd = []
	ver_bnd = []
	hor_sm = []
	ver_sm = []
# Arrays with horizontal and vertical components & labels of each sample
	hor = []
	ver = []
	lab = []

	print("Start filtering. Lowcut: ", lowcut, "Highcut: ", highcut, "Order: ", order)

# Read file by file
	for file_name in files:
		hor_tmp, ver_tmp, labels_tmp = read.read_file(file_name)

# Butterworth bandpas filter file by file
		sos = signal.butter(order, [lowcut, highcut], 'bandpass', analog=False, output='sos', fs=250)
		hor_bnd = signal.sosfilt(sos, hor_tmp)
		ver_bnd = signal.sosfilt(sos, ver_tmp)

# Butterworth bandpas filter file by file
#		[b,a] = signal.butter(order, [lowcut, highcut], 'bandpass', analog=False, output='ba', fs=250)
#		hor_bnd = signal.filtfilt(b,a, hor_tmp)
#		ver_bnd = signal.filtfilt(b,a, ver_tmp)

# Smoothing filter file by file
		hor_sm = signal.medfilt(hor_bnd, kernel_size = 35)
		ver_sm = signal.medfilt(ver_bnd, kernel_size = 35)
# Concatenate the results into ver and hor variables & labels
		hor.extend(hor_sm)
		ver.extend(ver_sm)
		lab.extend(labels_tmp)

# Split all data into trials & standarize over trials
	trial_hor, trial_ver, trial_lab = tri.to_trials(hor, ver, lab, samples_per_trial)

	return trial_hor, trial_ver, trial_lab