import matplotlib.pyplot as plt
import numpy as np
from matplotlib.collections import LineCollection


###################################################
# THIS FUNCTIONS ARE NOT INCLUDED IN THE PIPELINE #
###################################################
#   They are created as auxiliar functions that   #
#   could be called just in case they are needed  #
###################################################

def trial_plot(trial_hor, trial_ver, trial_lab,samples_per_trial):
	"""
# FUNCTION:	 	trial_plot(trial_hor, trial_ver, trial_lab)
# INPUT: 		Horizontal & Vertical components of each trial, its labels
# OUTPUT: 		Lineplot and number of samples per trial
# DESCRIPTION:	Generate the plots for horizontal and vertical values and
#				shows the division over samples
# TODO:			Add labels or colors based on labels
	"""

	hor = []
	ver = []
	
	for i in range(0, len(trial_lab)):
		for j in range(0, len(trial_hor[i])):
				hor.append(trial_hor[i][j])
				ver.append(trial_ver[i][j])
	
	plt.subplot(211)
	plt.plot(range(0, len(ver)), ver)
	plt.title(" Ver")
	plt.xticks(np.arange(0,len(ver), step=samples_per_trial))
	
	for j in range(0,len(ver), samples_per_trial):
		plt.axvline(x=j, color='tab:gray', linestyle=':')
	
	plt.subplot(212)
	plt.plot(range(0, len(hor)), hor)
	plt.title(" Hor")
	plt.xticks(np.arange(0,len(hor), step=samples_per_trial))
	
	for j in range(0,len(hor), samples_per_trial):
		plt.axvline(x=j, color='tab:gray', linestyle=':')
	plt.show()


def movement_plot(trial_hor, trial_ver, trial_lab, movement,samples_per_trial):
	"""
# FUNCTION:	 	movement_plot(trial_hor, trial_ver, trial_lab, movement)
# INPUT: 		Horizontal & Vertical components of each trial, its labels,
#				the movement to be shown & number of samples per trial
# OUTPUT: 		Lineplot
# DESCRIPTION:	Generate the plots for horizontal and vertical values and
#				shows the division over samples corresponding to the desired movement
	"""

	hor = []
	ver = []

	for i in range(0, len(trial_lab)):
		if trial_lab[i] == movement:
			for j in range(0, len(trial_hor[i])):
				hor.append(trial_hor[i][j])
				ver.append(trial_ver[i][j])

	plt.subplot(211)
	plt.plot(range(0, len(ver)), ver)
	plt.title(movement+" Ver")
	plt.xticks(np.arange(0,len(ver), step=samples_per_trial))
	for j in range(0,len(ver), samples_per_trial):
		plt.axvline(x=j, color='tab:gray', linestyle=':')
	
	plt.subplot(212)
	plt.plot(range(0, len(hor)), hor)
	plt.title(movement+" Hor")
	plt.xticks(np.arange(0,len(hor), step=samples_per_trial))
	
	for j in range(0,len(hor), samples_per_trial):
		plt.axvline(x=j, color='tab:gray', linestyle=':')
	plt.show()


def feature_plot(feat, trial_lab, to_plot):
	"""
# FUNCTION:	 	feature_plot(feat, trial_lab, to_plot)
# INPUT: 		Matrix with the features,array with labels and
#				the name of the feature to plot
# OUTPUT: 		Boxplot
# DESCRIPTION:	Creates a boxplot with a box for all values of the desired feature
#				splited into movements and horizontal & vertical components
	"""
	
	feature_list = ["Min","Max","Median","Mode","Kurtosis","Max_position","Min_position","AUC","First_q","Third_q","Interquartile", "Derivative_Slope"]
	feature_index = feature_list.index(to_plot)

	up_ver_feat = []
	down_ver_feat = []
	left_ver_feat = []
	right_ver_feat = []

	up_hor_feat = []
	down_hor_feat = []
	left_hor_feat = []
	right_hor_feat = []

	for iter in range(0, len(trial_lab)):
		if trial_lab[iter] == "'Up'":
			up_hor_feat.append(feat[iter][2*feature_index])
			up_ver_feat.append(feat[iter][2*feature_index+1])
		elif trial_lab[iter] == "'Down'":
			down_hor_feat.append(feat[iter][2*feature_index])
			down_ver_feat.append(feat[iter][2*feature_index+1])
		elif trial_lab[iter] == "'Left'":
			left_hor_feat.append(feat[iter][2*feature_index])
			left_ver_feat.append(feat[iter][2*feature_index+1])
		elif trial_lab[iter] == "'Right'":
			right_hor_feat.append(feat[iter][2*feature_index])
			right_ver_feat.append(feat[iter][2*feature_index+1])

	data_hor = [up_hor_feat,down_hor_feat,left_hor_feat,right_hor_feat]
	data_ver = [up_ver_feat,down_ver_feat,left_ver_feat,right_ver_feat]

	fig, ax = plt.subplots()

	plt.subplot(211)
	plot_labels = ["Up Hor", "Down Hor", "'Left Hor'", "'Right Hor'"]
	ax.set_xticklabels(plot_labels)
	ax.set_title("Horizontal")
	#ax.yticks(np.arange(-1,1))
	plt.boxplot(data_hor)

	plt.subplot(212)
	plot_labels = ["Up Ver", "Down Ver", "'Left Ver'", "'Right Ver'"]
	ax.set_xticklabels(plot_labels)
	ax.set_title("Vertical")
	#ax.yticks(np.arange(-1,1))
	plt.boxplot(data_ver)
	plt.show()
