from sklearn.model_selection import train_test_split
from sklearn.svm import SVC
from sklearn.model_selection import cross_val_score

def classification(feat, trial_lab, cross_validation):
	"""
# FUNCTION:	 	feature_plot(feat, to_plot)
# INPUT: 		Matrix with the features, labels for the trials and a boxplot
#				indicating if we want to do cross-validation or not
# OUTPUT: 		It returns the model and shows model accuracy
# DESCRIPTION:	Creates an SVM model to fit the test data if 'cross_validation' is False
#				or creates multiple SVM models if 'cross_validation' is True
	"""

# Train model and predict labels
	c = 0.75
	kernel = 'linear'
	gamma = 'scale'
	decision = 'ovr'
	cv = 5
	test_size = 0.33 # Used only if 'cross_validation' == False
		
	if cross_validation == True:
		# Cross Validation over all data
		print("Classification started. C: ", c, "CV: ", cv, "Decision", decision)
		clf = SVC(C=c, kernel=kernel, gamma=gamma, probability=True, decision_function_shape=decision)
		scores = cross_val_score(clf, feat, trial_lab, cv=cv)
		print("Mean Accuracy: %0.2f" % (scores.mean()))
		print("All accuracies: ", scores)
	else:
		# Split Features and Labels into train & test
		feat_train, feat_test, trial_lab_train, trial_lab_test = train_test_split(feat, trial_lab, test_size)
		
		# Classify over Train & Test data
		clf = SVC(C=c, kernel=kernel, gamma=gamma, probability=True, decision_function_shape=decision)
		clf.fit(feat_train, trial_lab_train)
		print("Accuracy: ", clf.score(feat_test, trial_lab_test))

	return clf